@@ -3,18 +3,19 @@
     vdirsyncer.exceptions
     ~~~~~~~~~~~~~~~~~~~~~
 
+    Contains exception classes used by vdirsyncer. Not all exceptions are here,
+    only the most commonly used ones.
+
     :copyright: (c) 2014 Markus Unterwaditzer
     :license: MIT, see LICENSE for more details.
 '''
 
 
 class Error(Exception):
-
     '''Baseclass for all errors.'''
 
 
 class PreconditionFailed(Error):
-
     '''
       - The item doesn't exist although it should
       - The item exists although it shouldn't
@@ -26,28 +27,12 @@ class PreconditionFailed(Error):
 
 
 class NotFoundError(PreconditionFailed):
-
     '''Item not found'''
 
 
 class AlreadyExistingError(PreconditionFailed):
-
     '''Item already exists'''
 
 
 class WrongEtagError(PreconditionFailed):
-
     '''Wrong etag'''
-
-
-class StorageError(Error):
-
-    '''Internal or initialization errors with storage.'''
-
-
-class SyncError(Error):
-    pass
-
-
-class SyncConflict(SyncError):
-    pass
