@@ -6,21 +6,21 @@
     :copyright: (c) 2014 Markus Unterwaditzer, Christian Geier and contributors
     :license: MIT, see LICENSE for more details.
 '''
-
-from .base import Storage, Item
-from .http import prepare_auth, prepare_verify, USERAGENT
-from .. import exceptions
-from .. import log
-from ..utils import request, get_password, urlparse
-import requests
 import datetime
+
 from lxml import etree
 
+import requests
+
+from .base import Item, Storage
+from .http import prepare_auth, prepare_verify, USERAGENT
+
+from .. import exceptions, log, utils
+
 
 dav_logger = log.get(__name__)
 
 CALDAV_DT_FORMAT = '%Y%m%dT%H%M%SZ'
-CONFIG_DT_FORMAT = '%Y-%m-%d'
 
 
 class DavStorage(Storage):
@@ -59,7 +59,7 @@ class DavStorage(Storage):
         super(DavStorage, self).__init__(**kwargs)
 
         if username and not password:
-            password = get_password(username, url)
+            password = utils.get_password(username, url)
 
         self._settings = {
             'verify': prepare_verify(verify),
@@ -70,9 +70,9 @@ class DavStorage(Storage):
 
         url = url.rstrip('/') + '/'
         if collection is not None:
-            url = urlparse.urljoin(url, collection)
+            url = utils.urlparse.urljoin(url, collection)
         self.url = url.rstrip('/') + '/'
-        self.parsed_url = urlparse.urlparse(self.url)
+        self.parsed_url = utils.urlparse.urlparse(self.url)
         self.collection = collection
 
         headers = self._default_headers()
@@ -84,7 +84,7 @@ class DavStorage(Storage):
         )
         response.raise_for_status()
         if self.dav_header not in response.headers.get('DAV', ''):
-            raise exceptions.StorageError('URL is not a collection')
+            raise ValueError('URL is not a collection')
 
     def _default_headers(self):
         return {
@@ -104,7 +104,7 @@ class DavStorage(Storage):
             ssl_verify=kwargs.get('verify', True)
         )
         for c in d.discover():
-            collection = urlparse.urljoin(url, c['href'])
+            collection = utils.urlparse.urljoin(url, c['href'])
             if collection.startswith(url):
                 collection = collection[len(url):]
             collection = collection.rstrip('/')
@@ -115,12 +115,15 @@ class DavStorage(Storage):
     def _normalize_href(self, href):
         '''Normalize the href to be a path only relative to hostname and
         schema.'''
-        x = urlparse.urljoin(self.url, href)
+        if not href:
+            raise ValueError(href)
+        x = utils.urlparse.urljoin(self.url, href)
         assert x.startswith(self.url)
-        return urlparse.urlsplit(x).path
+        return utils.urlunquote_plus(utils.urlparse.urlsplit(x).path)
 
-    def _get_href(self, uid):
-        return self._normalize_href(super(DavStorage, self)._get_href(uid))
+    def _get_href(self, item):
+        href = utils.urlunquote_plus(item.ident) + self.fileext
+        return self._normalize_href(href)
 
     def _request(self, method, path, data=None, headers=None):
         path = path or self.parsed_url.path
@@ -128,8 +131,8 @@ class DavStorage(Storage):
         if self._session is None:
             self._session = requests.session()
         url = self.parsed_url.scheme + '://' + self.parsed_url.netloc + path
-        return request(method, url, data=data, headers=headers,
-                       session=self._session, **self._settings)
+        return utils.request(method, url, data=data, headers=headers,
+                             session=self._session, **self._settings)
 
     @staticmethod
     def _check_response(response):
@@ -217,7 +220,7 @@ class DavStorage(Storage):
         return self._put(href, item, etag)
 
     def upload(self, item):
-        href = self._get_href(item.uid)
+        href = self._get_href(item)
         return self._put(href, item, None)
 
     def delete(self, href, etag):
