@@ -8,10 +8,12 @@
 '''
 
 import os
-from vdirsyncer.storage.base import Storage, Item
+
+from .base import Item, Storage
 import vdirsyncer.exceptions as exceptions
-from vdirsyncer.utils import expand_path, text_type
 import vdirsyncer.log as log
+from vdirsyncer.utils import expand_path, text_type
+
 logger = log.get(__name__)
 
 
@@ -98,9 +100,6 @@ class FilesystemStorage(Storage):
     def _get_filepath(self, href):
         return os.path.join(self.path, href)
 
-    def _get_href(self, uid):
-        return uid + self.fileext
-
     def list(self):
         for fname in os.listdir(self.path):
             fpath = os.path.join(self.path, fname)
@@ -121,10 +120,10 @@ class FilesystemStorage(Storage):
                 raise
 
     def upload(self, item):
-        href = self._get_href(item.uid)
+        href = self._get_href(item)
         fpath = self._get_filepath(href)
         if os.path.exists(fpath):
-            raise exceptions.AlreadyExistingError(item.uid)
+            raise exceptions.AlreadyExistingError(item)
 
         if not isinstance(item.raw, text_type):
             raise TypeError('item.raw must be a unicode string.')
@@ -135,7 +134,7 @@ class FilesystemStorage(Storage):
 
     def update(self, href, item, etag):
         fpath = self._get_filepath(href)
-        if href != self._get_href(item.uid):
+        if href != self._get_href(item) and item.uid:
             logger.warning('href != uid + fileext: href={}; uid={}'
                            .format(href, item.uid))
         if not os.path.exists(fpath):
