@@ -7,22 +7,23 @@
     :license: MIT, see LICENSE for more details.
 '''
 
-import hashlib
-from .base import Storage, Item
-from vdirsyncer.utils import expand_path, get_password, request, urlparse, \
-    text_type
+from .base import Item, Storage
+from ..utils import expand_path, get_password, request, text_type, urlparse
 
 USERAGENT = 'vdirsyncer'
 
 
-def split_collection(text):
+def split_simple_collection(lines):
     item = []
     collection_type = None
     item_type = None
-    for line in text.splitlines():
-        if not line.strip():
-            continue
-        key, value = (x.strip() for x in line.split(u':', 1))
+    for line in lines:
+        if u':' not in line:
+            key = line
+            value = None
+        else:
+            key, value = (x.strip() for x in line.split(u':', 1))
+
         if key == u'BEGIN':
             if collection_type is None:
                 collection_type = value
@@ -36,8 +37,9 @@ def split_collection(text):
                 break
             elif value == item_type:
                 item.append(line)
-                yield Item(u'\n'.join(item))
+                yield item
                 item = []
+                item_type = None
             else:
                 item.append(line)
         else:
@@ -45,6 +47,51 @@ def split_collection(text):
                 item.append(line)
 
 
+def wrap_items(items, collection_type, exclude=(u'VTIMEZONE',)):
+    for item in items:
+        key, value = (x.strip() for x in item[0].split(u':'))
+        if value in exclude:
+            yield item
+        else:
+            yield ([u'BEGIN:' + collection_type] + item +
+                   [u'END:' + collection_type])
+
+
+def inline_timezones(items):
+    timezone = None
+    for item in items:
+        if u':' not in item[0]:
+            import pdb
+            pdb.set_trace()
+
+        key, value = (x.strip() for x in item[0].split(u':'))
+        if value == u'VTIMEZONE':
+            if timezone is not None:
+                raise ValueError('Multiple timezones.')
+            timezone = item
+        else:
+            if timezone is not None:
+                item = [item[0]] + timezone + item[1:]
+            yield item
+
+
+def split_collection(lines):
+    collection_type = None
+    for line in lines:
+        key, value = (x.strip() for x in line.split(u':'))
+        if key == u'BEGIN':
+            collection_type = value
+            break
+
+    is_calendar = collection_type == u'VCALENDAR'
+    rv = split_simple_collection(lines)
+
+    if is_calendar:
+        rv = inline_timezones(wrap_items(rv, collection_type))
+
+    return rv
+
+
 def prepare_auth(auth, username, password):
     if (username and password) or auth == 'basic':
         return (username, password)
@@ -85,7 +132,8 @@ class HttpStorage(Storage):
 
         self._settings = {
             'verify': prepare_verify(verify),
-            'auth': prepare_auth(auth, username, password)
+            'auth': prepare_auth(auth, username, password),
+            'latin1_fallback': False
         }
         self.username, self.password = username, password
         self.useragent = useragent
@@ -104,13 +152,12 @@ class HttpStorage(Storage):
         r = request('GET', self.url, **self._settings)
         r.raise_for_status()
         self._items.clear()
-        for i, item in enumerate(split_collection(r.text)):
-            uid = item.uid if item.uid is not None else i
-            self._items[uid] = item
+        for i, item in enumerate(split_collection(r.text.splitlines())):
+            item = Item(u'\n'.join(item))
+            self._items[self._get_href(item)] = item, item.hash
 
-        for uid, item in self._items.items():
-            yield uid, hashlib.sha256(item.raw.encode('utf-8')).hexdigest()
+        for href, (item, etag) in self._items.items():
+            yield href, etag
 
     def get(self, href):
-        x = self._items[href]
-        return x, hashlib.sha256(x.raw.encode('utf-8')).hexdigest()
+        return self._items[href]
