@@ -22,11 +22,16 @@ PY2 = sys.version_info[0] == 2
 
 if PY2:
     import urlparse
+    from urllib import \
+         quote_plus as urlquote_plus, \
+         unquote_plus as urlunquote_plus
     text_type = unicode  # flake8: noqa
     iteritems = lambda x: x.iteritems()
     itervalues = lambda x: x.itervalues()
 else:
     import urllib.parse as urlparse
+    urlquote_plus = urlparse.quote_plus
+    urlunquote_plus = urlparse.unquote_plus
     text_type = str
     iteritems = lambda x: x.items()
     itervalues = lambda x: x.values()
@@ -59,8 +64,28 @@ def split_dict(d, f):
     return a, b
 
 
-def parse_options(items):
+def parse_options(items, section=None):
     for key, value in items:
+        if len(value.splitlines()) > 1:
+            # The reason we use comma-separated values instead of
+            # multiline-values for lists is simple: ConfigParser's barrier for
+            # mistaking an arbitrary line for the continuation of a value is
+            # awfully low.
+            #
+            # Take this example:
+            #
+            # foo = bar
+            #  # my comment
+            #
+            # For reasons beyond my understanding ConfigParser only requires
+            # one space to interpret the line as part of a multiline-value.
+            # Related to that, it also parses any inline-comments as value:
+            #
+            # foo = bar  # this comment is part of the value!
+            raise ValueError('Section {!r}, option {!r}: '
+                             'No multiline-values allowed.'
+                             .format(section, key))
+
         if value.lower() in ('yes', 'true', 'on'):
             value = True
         elif value.lower() in ('no', 'false', 'off'):
@@ -167,7 +192,7 @@ def get_password(username, resource):
 
 
 def request(method, url, data=None, headers=None, auth=None, verify=None,
-            session=None):
+            session=None, latin1_fallback=True):
     '''wrapper method for requests, to ease logging and mocking'''
 
     if session is None:
@@ -180,7 +205,17 @@ def request(method, url, data=None, headers=None, auth=None, verify=None,
     logger.debug(data)
     logger.debug('Sending request...')
     r = func(method, url, data=data, headers=headers, auth=auth, verify=verify)
+
+    # See https://github.com/kennethreitz/requests/issues/2042
+    content_type = r.headers.get('Content-Type', '')
+    if not latin1_fallback and \
+       'charset' not in content_type and \
+       content_type.startswith('text/'):
+        logger.debug('Removing latin1 fallback')
+        r.encoding = None
+
     logger.debug(r.status_code)
     logger.debug(r.headers)
     logger.debug(r.content)
+
     return r
